/*
Adapted from original arduino pendant2 from ArcEye
by Gerd Gleixner
12.08.2023
*/
//#include <SoftwareSerial.h>
#include <Key.h>
#include <Keypad.h>
//#include "Wire.h"
//#include "LiquidCrystal.h"
#include <LiquidCrystal_I2C.h>

#define I2C_LCD  0x27   // 0x20 - 0x27

// predefs
void sendCommand(char);
void noKey();
void zeroX();
void zeroY();
void zeroZ();
void zeroS();

/*  keyboard layout

     C1 C2 C3 C4 C5
 R1  0  1  2  3  4
 R2  5  6  7  8  9
 R3  A  B  C  D  E 
 R4  F  G  H  I  J

 0,5,A   = X, Y, Z
 4,9,E,J = ON/OFF, Man, MDI, Auto
 1,6,B,G = jogging

*/

#define ROWS 4
#define COLS 5

// keypad layout
/* old 4x4
char padKeys[ROWS][COLS] = { {'0','1','2','3'}, {'4','5','6','7'}, {'8','9','A','B'}, {'C','D','E','F'} }; 
byte rowPins[ROWS] = {2, 3, 4, 5}; // The Arduino pins to connect to the row pins of the keypad 
byte colPins[COLS] = {9, 8, 7, 6}; // The Arduino pins to connect to the column pins of the keypad 
*/
char padKeys[ROWS][COLS] = { {'0','1','2','3','4'}, {'5','6','7','8','9'}, {'A','B','C','D','E'}, {'F','G','H','I','J'} }; 
byte rowPins[ROWS] = {9, 10, 11, 12}; // The Arduino pins to connect to the row pins of the keypad 
byte colPins[COLS] = {8, 7, 6, 5, 4}; // The Arduino pins to connect to the column pins of the keypad 

Keypad ourKeypad = Keypad( makeKeymap(padKeys), rowPins, colPins, ROWS, COLS); // Set up an instance of the keypad 

LiquidCrystal_I2C  lcd (0x27,2,1,0,4,5,6,7,3, POSITIVE); // Set LCD I2C address and pinout

char XReading[9];  // 0000.000\0 pos
char YReading[9];
char ZReading[9];
char SReading[17];   // Status

char oldXReading[9];
char oldYReading[9];
char oldZReading[9];
char oldSReading[17] = "0123456789ABCDEF";

void setup() 
{
    Serial.begin(115200);

        // set up the LCD's number of rows and columns: 
    lcd.begin(20, 4);
    lcd.setBacklight(HIGH);
    lcd.noAutoscroll();
        // set static information
    lcd.setCursor(0, 0);
    lcd.print("X*");
    lcd.setCursor(0, 1);
    lcd.print("Y ");
    lcd.setCursor(0, 2);
    lcd.print("Z ");
    lcd.setCursor(0, 3);
    lcd.print("S ");
   
    zeroX();
    zeroY();
    zeroZ();
    zeroS();
 
    ourKeypad.setDebounceTime(10);

}

void loop() 
{
int x ;
volatile KeyState keystate, laststate;
volatile char ch, key, lastkey;
 
//************************** display data form CNC ******************

    if(Serial.available() >= 9)   // Data from CNC available
        {
        ch = Serial.read();
        switch(ch)
            {
            case 'X':
                    while(Serial.available() < 8)
                        delay(10);
                    for(x = 0; x < 8; x++)
                        XReading[x] = Serial.read();
                    if(strncmp(XReading, oldXReading, 8) != 0)
                        {
                        lcd.setCursor(2, 0);
                        lcd.print(XReading);
                        strcpy(oldXReading, XReading);
                        }
                    zeroX();
                    break;                    
            
            case 'Y':
                    while(Serial.available() < 8)
                        delay(10);
                    for(x = 0; x < 8; x++)
                        YReading[x] = Serial.read();
                    if(strncmp(YReading, oldYReading, 8) != 0)
                        {
                        lcd.setCursor(2, 1);
                        lcd.print(YReading);
                        strcpy(oldYReading, YReading);
                        }
                    zeroY();
                    break;                    

            case 'Z':
                    while(Serial.available() < 8)
                        delay(10);
                    for(x = 0; x < 8; x++)
                        ZReading[x] = Serial.read();
                    if(strncmp(ZReading, oldZReading, 8) != 0)
                        {    
                        lcd.setCursor(2, 2);
                        lcd.print(ZReading);
                        strcpy(oldZReading, ZReading);
                        }
                    zeroZ();
                    break;
  
            case 'S':
                    while(Serial.available() < 16)
                        delay(10);
                    for(x = 0; x < 16; x++)
                        SReading[x] = Serial.read();
                    if(strncmp(SReading, oldSReading, 16) != 0)
                        {    
                        lcd.setCursor(2, 3);
                        lcd.print(SReading);
                        strcpy(oldSReading, SReading);
// LCD 1st line ON/OFF and mode
                        lcd.setCursor(11, 0);  // Machine on/off
                        if (SReading[0]=='1') lcd.print("ON ");
                        else                  lcd.print("OFF");
                        lcd.setCursor(15, 0);  // Machine mode
                        if (SReading[1]=='1') lcd.print("Man ");
                        if (SReading[2]=='1') lcd.print("MDI ");
                        if (SReading[3]=='1') lcd.print("Auto");
// LCD 2nd line Mist and Spindel
                        lcd.setCursor(10, 1);  // Mist on/off
                        if (SReading[4]=='1') lcd.print("Mist");
                        else                  lcd.print("    ");
                        lcd.setCursor(15, 1);  // Spindel on/off
                        if (SReading[5]=='1') lcd.print("Spin");
                        else                  lcd.print("    ");
// LCD 3rd line program status
                        lcd.setCursor(10, 2);  // Run/Pause
                        if (SReading[6]=='1')      lcd.print("Run  ");
                        else if (SReading[7]=='1') lcd.print("Pause");
                        else                       lcd.print("     ");
// active axis
                        if (SReading[8]=='1') {   // X Achse
                          lcd.setCursor(1, 0);
                          lcd.print("*");
                          lcd.setCursor(1, 1);
                          lcd.print(" ");
                          lcd.setCursor(1, 2);
                          lcd.print(" "); }
                        if (SReading[9]=='1') {   // Y Achse
                          lcd.setCursor(1, 0);
                          lcd.print(" ");
                          lcd.setCursor(1, 1);
                          lcd.print("*");
                          lcd.setCursor(1, 2);
                          lcd.print(" "); }
                        if (SReading[10]=='1') {   // Z Achse
                          lcd.setCursor(1, 0);
                          lcd.print(" ");
                          lcd.setCursor(1, 1);
                          lcd.print(" ");
                          lcd.setCursor(1, 2);
                          lcd.print("*"); }
                        }  
                        zeroS();
                    break;
            default:
                    break;
            }
        }
 
    // ***************** key-pad processing ********************

 //   if (ourKeypad.keyStateChanged()) 
     
      key = ourKeypad.getKey();        // input from keyboard ?
      keystate = ourKeypad.getState(); // state from keyboard

     // if(key == NO_KEY) key = 'X'; // replace no-key by 'X'

      if(key != NO_KEY) {  // key pressed
        Serial.write(key); // write command
        lcd.setCursor(19, 3);
        lcd.print(key); 
        lastkey = key;
      }
      //lcd.setCursor(18, 3);
      //lcd.print(lastkey); 
    
    if (laststate != keystate ) {
      switch (keystate) {
        case IDLE :
        break;
        case PRESSED :
        break;
        case RELEASED :
         Serial.write('X');      // reset commands
        break;
        case HOLD :
        break;
        default : 
        break;
      }
    }
  
    laststate = keystate;
        
}
//#############################################################################################################

void zeroX() { for(int x = 0; x <  9; x++) XReading[x] = 0;}
void zeroY() { for(int x = 0; x <  9; x++) YReading[x] = 0;}
void zeroZ() { for(int x = 0; x <  9; x++) ZReading[x] = 0;}
void zeroS() { for(int x = 0; x < 17; x++) SReading[x] = 0;}

